import django_rtk.mutations as mutations
import graphene
from django_rtk.utils import (
    extract_token,
    get_backend,
    get_errors,
    get_setting_or,
    get_validator,
)


class RegisterAccount(mutations.RegisterAccount):
    class Arguments:
        email = graphene.String()
        password = graphene.String()
        accepts_terms = graphene.Boolean()
        terms_version_accepted = graphene.String(
            required=False, default_value=get_setting_or("1.0.0", "TERMS_VERSION")
        )
        if get_setting_or(False, "REQUIRE_USERNAME"):
            username = graphene.String()

    activation_token = graphene.String()

    @classmethod
    def validate_args(cls, errors, email, password, **kwargs):
        get_validator().validate_email(errors, email)
        get_validator().validate_password(errors, password)
        if get_setting_or(False, "REQUIRE_USERNAME"):
            get_validator().validate_username(errors, kwargs["username"])

    @classmethod
    def run(cls, errors, **kwargs):
        result = get_backend().register_account(errors, **kwargs)

        hide_account_existence = get_setting_or(True, "HIDE_ACCOUNT_EXISTENCE")
        if hide_account_existence and "ALREADY_TAKEN" in get_errors(errors, "email"):
            get_errors(errors, "email").remove("ALREADY_TAKEN")
            mutations.send_registered_again_email(result, **kwargs)

        return result

    @classmethod
    def get_output_values(cls, result):
        return {"activation_token": extract_token(result, "activation_token")}

    @classmethod
    def send_email(cls, result, **kwargs):
        if result.get("activation_token"):
            mutations.send_activation_email(result, **kwargs)



import graphene

from .activateaccount import ActivateAccount
from .registeraccount import RegisterAccount


class Mutation(graphene.ObjectType):
    register_account = RegisterAccount.Field()
    activate_account = ActivateAccount.Field()



# Generated by Django 4.0.5 on 2022-06-25 06:34

from django.db import migrations, models
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='ActivationToken',
            fields=[
                ('email', models.CharField(max_length=255, primary_key=True, serialize=False)),
                ('token', models.UUIDField(default=uuid.uuid4, editable=False, unique=True)),
                ('created', models.DateField(auto_now_add=True)),
            ],
        ),
    ]



import uuid

from django.db import models


class ActivationToken(models.Model):
    email = models.CharField(max_length=255, primary_key=True)
    token = models.UUIDField(default=uuid.uuid4, editable=False, unique=True)
    created = models.DateField(auto_now_add=True)



import uuid

from django.contrib.auth import get_user_model
from django_rtk.utils import count_errors, get_errors

from django_rtk_upfront.models import ActivationToken


class Backend:
    def register_account(
        self, errors, email, password, accepts_terms, terms_version_accepted, **kwargs
    ):
        result = dict(activation_token="")
        if count_errors(errors):
            return result

        if get_user_model().objects.filter(email=email).exists():
            get_errors(errors, "email").append("ALREADY_TAKEN")
            return result

        user = result["user"] = get_user_model().objects.create_user(
            email=email,
            password=password,
            username=kwargs.pop("username", email),
            accepts_terms=accepts_terms,
            terms_version_accepted=terms_version_accepted,
            **kwargs,
        )
        user.is_active = False
        user.save()

        activation_token, _ = ActivationToken.objects.get_or_create(email=email)
        result["activation_token"] = activation_token.token.hex
        return result

    def activate_account(self, errors, activation_token, **kwargs):
        result = dict()
        if count_errors(errors):
            return result

        activation_token = ActivationToken.objects.filter(
            token=uuid.UUID(activation_token.token.hex)
        ).first()
        if activation_token:
            user = get_user_model().objects.filter(email=activation_token.email).first()
            activation_token.delete()
        else:
            user = None

        if user:
            user.is_active = True
            user.save()
        else:
            get_errors(errors, "activation_token").append("ACCOUNT_UNKNOWN")

        return result



from django.contrib import admin

from django_rtk_upfront import models


class ActivationTokenAdmin(admin.ModelAdmin):
    readonly_fields = ["token"]


admin.site.register(models.ActivationToken, ActivationTokenAdmin)



from setuptools import find_packages, setup

setup(
    name="django-rtk-upfront",
    version="0.6.0",
    description="Extension of django-rtk that requires a password in the registration step",
    long_description="Extension of django-rtk that requires a password in the registration step",
    long_description_content_type="text/x-rst",
    url="https://github.com/mnieber/django-rtk-home/blob/main/django-rtk-upfront/README.md",
    author="Maarten Nieber",
    author_email="hallomaarten@yahoo.com",
    license="MIT",
    packages=find_packages(),
    package_data={},
    entry_points={},
    data_files=[],
    cmdclass={},
    install_requires=["django-rtk"],
    zip_safe=False,
    python_requires=">=3.6",
)
